﻿#pragma warning( disable : 4996 )
#include "CWindow.h"

///////////////////////////////////////////////////////////////////////////////////
// スタティック変数
///////////////////////////////////////////////////////////////////////////////////
HINSTANCE			CWindow::hInstance		= NULL;
HWND				CWindow::hWnd			= NULL;
BOOL				CWindow::bActive		= TRUE;

WCHAR				CWindow::mName[256]		= L"";
const WCHAR*		CWindow::cIconID		= IDI_APPLICATION;		// デフォルトのアイコン
HMENU				CWindow::hMenu			= NULL;
DWORD				CWindow::dwStyle		= WS_POPUP|WS_SYSMENU|WS_CAPTION|WS_MINIMIZEBOX;
DWORD				CWindow::dwExStyle		= 0;

LPCWONMSGLIST		CWindow::pMsgList		= NULL;
int					CWindow::iMsgList		= 0;


///////////////////////////////////////////////////////////////////////////////////
// コンストラクタ
///////////////////////////////////////////////////////////////////////////////////
CWindow::CWindow(void)
{

}

///////////////////////////////////////////////////////////////////////////////////
// デストラクタ
///////////////////////////////////////////////////////////////////////////////////
CWindow::~CWindow()
{

}

///////////////////////////////////////////////////////////////////////////////////
// メインウインドウのイベントハンドラ
///////////////////////////////////////////////////////////////////////////////////
LRESULT CALLBACK CWindow::WindowProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	static CWindow *win = NULL;

	switch( uMsg )
	{
	case WM_CREATE:
		win = (CWindow*)lParam;
		break;
	case WM_ACTIVATE:
		bActive = LOWORD(wParam)?TRUE:FALSE;				// アクティブ状態変更
		break;
	case WM_DESTROY:										// ALT+F4が押されたら
		PostQuitMessage( 0 );
		break;
	case WM_MOUSEMOVE:
		break;
	case WM_SYSCOMMAND:
		switch( wParam )
		{
		case SC_CLOSE:
			PostQuitMessage(0);
			return 0;
		}
		break;
	case WM_IME_NOTIFY:
		switch( wParam )
		{
		case IMN_SETOPENSTATUS:
			HIMC hImc = ImmGetContext( hWnd );
            ImmSetOpenStatus( hImc,FALSE );
			break;
		}
		break;
    }

	// 特殊メッセージ処理
	for( int i=0;i<iMsgList;i++ ) {
		if( uMsg==pMsgList[i].dwMsg ) {
			return pMsgList[i].pProc( hWnd,wParam,lParam );		// 特殊メッセージ操作完了なら
		}
	}

	return DefWindowProc( hWnd,uMsg,wParam,lParam );		// デフォルトを返す
}

///////////////////////////////////////////////////////////////////////////////////
// ウインドウを生成する
///////////////////////////////////////////////////////////////////////////////////
BOOL CWindow::Create( HINSTANCE hInst,const WCHAR *appName,BOOL show,DWORD w,DWORD h,HWND parent )
{
	WNDCLASSW wc;
	DEVMODEW dmMode;

	// 画面解像度をチェック
	EnumDisplaySettingsW( NULL,ENUM_CURRENT_SETTINGS,&dmMode );
	// 16bit以上の解像度じゃないと起動できない
	if( dmMode.dmBitsPerPel<16 ) {
		MessageBoxW( GetDesktopWindow(),L"16Bit以上の解像度にしてください",L"起動できません",MB_OK );
		return FALSE;
	}

	// セット
	hInstance = hInst;
	wcscpy( mName,appName );

	// ウインドウクラス登録
	wc.style			= CS_HREDRAW | CS_VREDRAW | CS_BYTEALIGNCLIENT;
    wc.lpfnWndProc		= WindowProc;
    wc.cbClsExtra		= 0;
    wc.cbWndExtra		= sizeof(DWORD);
    wc.hInstance		= hInstance;
    wc.hIcon			= LoadIcon(hInstance, cIconID );
    wc.hCursor			= LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground	= (HBRUSH)GetStockObject(WHITE_BRUSH);
	wc.lpszMenuName		= MAKEINTRESOURCEW( hMenu );
    wc.lpszClassName	= mName;
    if( !RegisterClassW(&wc) )
        return FALSE;

	// ウインドウ生成
    hWnd = CreateWindowExW(
		dwExStyle,
		wc.lpszClassName,						// Class
		mName,									// Title bar
		dwStyle,								// Style
		GetSystemMetrics(SM_CXSCREEN)/2-w/2,
		GetSystemMetrics(SM_CYSCREEN)/2-h/2,
		w,										// Init. x pos
		h,										// Init. y pos
		parent,									// Parent window
		NULL,									// Menu handle
		hInstance,								// Program handle
		this									// Create parms
	);
    if( !hWnd )
        return FALSE;							// 生成に失敗

	// フォントの設定
	HDC hdc = GetDC( hWnd );
	if( hdc ) {
		SetBkMode( hdc,TRANSPARENT );
		ReleaseDC( hWnd,hdc );
	}

	// クライアントエリアのサイズに合わせたウィンドウサイズを算出し、画面の中心に表示する
	MoveClientWindowCenter( w,h );

	// ウインドウを表示
	if( show )
		::ShowWindow( hWnd,SW_SHOW );

	return TRUE;
}

///////////////////////////////////////////////////////////////////////////////////
// 明示的にウィンドウを削除する
///////////////////////////////////////////////////////////////////////////////////
void CWindow::Delete( void )
{
	if( hWnd ) {
		// 通常のウィンドウなら
		::DestroyWindow( hWnd );
		// 登録したクラス名を解除
		UnregisterClassW( mName,hInstance );
		ZeroMemory( &mName,sizeof(mName) );
		hWnd = NULL;
		hInstance = NULL;
	}
}

///////////////////////////////////////////////////////////////////////////////////
// カーソルの表示・非表示
///////////////////////////////////////////////////////////////////////////////////
void CWindow::ShowCursor( BOOL bShow )
{
	if( bShow )
		while(::ShowCursor(TRUE)<0);
	else
		while(::ShowCursor(FALSE)>=0);
}

///////////////////////////////////////////////////////////////////////////////////
// ウインドウの表示・非表示
///////////////////////////////////////////////////////////////////////////////////
void CWindow::ShowWindow( BOOL bShow )
{
	// ウインドウの表示
	if( bShow )
		::ShowWindow( hWnd,SW_SHOW );
	else
		::ShowWindow( hWnd,SW_HIDE );
}

///////////////////////////////////////////////////////////////////////////////////
// アプリケーションのアイコンの変更
///////////////////////////////////////////////////////////////////////////////////
void CWindow::SetIcon( const WCHAR *icon )
{
	cIconID = icon;
}


// 特殊メッセージの追加
BOOL CWindow::AddMsgProc( UINT msg,LPONMESSAGE proc )
{
	// 既に存在していないかチェック
	for( int i=0;i<iMsgList;i++ ) {
		if( pMsgList[i].dwMsg==msg ) {
			// あれば新しいアドレスに更新
			pMsgList[i].pProc = proc;
			return TRUE;
		}
	}

	// 追加
	iMsgList++;
	pMsgList = (LPCWONMSGLIST)realloc( pMsgList,sizeof(CWONMSGLIST)*iMsgList );
	ZeroMemory( &pMsgList[iMsgList-1],sizeof(CWONMSGLIST) );
	pMsgList[iMsgList-1].dwMsg	= msg;
	pMsgList[iMsgList-1].pProc	= proc;
	return TRUE;
}

// ウインドウスタイルの変更(動的に変更も可能)
BOOL CWindow::SetWindowStyle( DWORD style )
{
	dwStyle = style;
	if( hWnd ) {
		// すでにウインドウが存在する場合は即反映
		::SetWindowLongW( hWnd,GWL_STYLE,style );
		::SetWindowPos( hWnd,0,0,0,0,0,SWP_FRAMECHANGED|SWP_NOACTIVATE|SWP_NOMOVE|SWP_NOOWNERZORDER|SWP_NOSIZE|SWP_NOZORDER );
	}
	return TRUE;
}

// ウインドウの移動
void CWindow::Move( int x,int y )
{
	SetWindowPos( hWnd,0,x,y,0,0,SWP_NOSIZE|SWP_NOZORDER );
}

// ウインドウの移動(幅と高さも同時に変更)
void CWindow::Move( int x,int y,int w,int h )
{
	MoveWindow( hWnd,x,y,w,h,TRUE );
}

// 指定サイズがクライアント領域になるようにウインドウを配置
BOOL CWindow::MoveClientWindowCenter( int w,int h )
{
	// スタイルに合わせたウインドウサイズを算出
	RECT rect = { 0,0,w,h };
	AdjustWindowRect(&rect, dwStyle, hMenu ? TRUE : FALSE);
	
	// センター位置を算出
	int scr_w = GetSystemMetrics( SM_CXSCREEN );			// スクリーンの幅
	int scr_h = GetSystemMetrics( SM_CYSCREEN );			// スクリーンの高さ
	int win_w = rect.right - rect.left;
	int win_h = rect.bottom - rect.top;
	int x = ( scr_w - win_w ) / 2;
	int y = ( scr_h - win_h ) / 2;

	// 移動とサイズを同時にセット
	SetWindowPos( hWnd,0,x,y,win_w,win_h,SWP_NOZORDER );

	return TRUE;
}

// メニューアイテムの変更
BOOL CWindow::SetMenuItem(int menuid,BOOL check,BOOL gray )
{
	HMENU menu = GetMenu( hWnd );
	if( menu ) {
		MENUITEMINFOW info;
		ZeroMemory( &info,sizeof(info) );
		info.cbSize	= sizeof(info);
		info.fMask	= MIIM_STATE;
		if( check )
			info.fState |= MFS_CHECKED;
		else
			info.fState |= MFS_UNCHECKED;
		if( gray )
			info.fState |= MFS_GRAYED;
		else
			info.fState |= MFS_ENABLED;
		return SetMenuItemInfoW( menu,menuid,FALSE,&info);
	}
	return TRUE;
}

BOOL CWindow::TextOutW( int x,int y,const WCHAR *str,COLORREF col )
{
	HDC hdc = GetDC( hWnd );
	if( hdc ) {
		SetTextColor( hdc,col );
		::TextOutW( hdc,x,y,str,(int)wcslen(str) );
		ReleaseDC( hWnd,hdc );
	}
	return TRUE;
}

///////////////////////////////////////////////////////////////////////////////////
// メニューバーの変更(Createの前に必要）
///////////////////////////////////////////////////////////////////////////////////
void CWindow::SetMenu( HMENU menu )
{
	hMenu = menu;
}
